@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,17 +26,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkfilesel.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 
 #include "claws.h"
 #include "main.h"
@@ -82,6 +72,7 @@ gint import_mbox(FolderItem *default_dest)
 		gtk_widget_show(window);
 	}
 
+	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	change_dir(claws_get_startup_dir());
 
 	if (default_dest && default_dest->path) {
@@ -102,6 +93,7 @@ gint import_mbox(FolderItem *default_dest)
 	gtk_main();
 
 	gtk_widget_hide(window);
+	gtk_window_set_modal(GTK_WINDOW(window), FALSE);
 
 	return import_ok;
 }
@@ -120,7 +112,6 @@ static void import_create(void)
 	gtk_window_set_title(GTK_WINDOW(window), _("Import mbox file"));
 	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(delete_event), NULL);
@@ -236,6 +227,7 @@ static void import_ok_cb(GtkWidget *widget, gpointer data)
 	if (!dest) {
 		alertpanel_error(_("Can't find the destination folder."));
 		gtk_widget_grab_focus(dest_entry);
+		g_free(mbox);
 		return;
 	} else {
 		import_ok = proc_mbox(dest, mbox, FALSE, NULL);
@@ -275,7 +267,7 @@ static void import_destsel_cb(GtkWidget *widget, gpointer data)
 	FolderItem *dest;
 	gchar *path;
 
-	dest = foldersel_folder_sel(NULL, FOLDER_SEL_COPY, NULL);
+	dest = foldersel_folder_sel(NULL, FOLDER_SEL_COPY, NULL, FALSE);
 	if (!dest)
 		 return;
 	path = folder_item_get_identifier(dest);
@@ -291,7 +283,7 @@ static gint delete_event(GtkWidget *widget, GdkEventAny *event, gpointer data)
 
 static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (event && event->keyval == GDK_Escape)
+	if (event && event->keyval == GDK_KEY_Escape)
 		import_cancel_cb(NULL, NULL);
 	return FALSE;
 }
